/*
 Copyright (C) 2007 by Ben Anderman <ben@happyspork.com>
 Part of Sporktris http://www.happyspork.com/games
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY.
 
 See the License.txt file for more details.
 */

#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <cstdio>
#include <stdlib.h>
#include <vector>
#include <string>
#include <fstream>

#include "menu.h"
#include "namedv.h"
#include "prefs_file.h"
#include "common_funcs.h"

using namespace std;

string Menu::hit_test (void)
{
	ls.first = true;
	int x, y;
	SDL_GetMouseState(&x, &y);
	handle_mouse_move(x, y);
	screen = SDL_GetVideoSurface();
	ls.blit(screen);
	SDL_Flip(screen);
	SDL_Event event;
	while (SDL_WaitEvent(&event) != 0)
	{
		if (event.type == SDL_MOUSEBUTTONDOWN)
		{
			for (unsigned int i = 0; i < button_rects.size(); i++)
			{
				SDL_MouseButtonEvent temp = event.button;
				if (temp.x > button_rects[i].x && temp.x < button_rects[i].x+button_rects[i].w && temp.y > button_rects[i].y && temp.y < button_rects[i].y+button_rects[i].h)
				{
					string tmp = menu_info[string("button")+funcs::str_int(i+1)+string("_return")];
					if (tmp != "") return (tmp);
				}
			}
		} else if (event.type == SDL_MOUSEMOTION) {
			SDL_MouseMotionEvent mEvent = event.motion;
			bool udWin = false;
			udWin = handle_mouse_move(mEvent.x, mEvent.y);
			if (udWin)
			{
				ls.blit(screen);
				SDL_Flip(screen);
			}
		} else if (event.type == SDL_QUIT) {
			return("quit");
		}
	}
	return ("");
}

bool Menu::handle_mouse_move(int x, int y)
{
	int num_buttons = menu_info.geti("num_buttons");
	bool result = false;
	for (int i = 1; i <= num_buttons; i++)
	{
		string temp1 = string("button")+funcs::str_int(i);
		if (menu_info[temp1+string("_surf_over")] != "" && surfs[menu_info[temp1+string("_surf_over")]] != NULL)
		{
			layer *l = &(ls.layers[i]);
			if (hit_test(x, y, &(button_rects[i-1])))
			{
				if (l->surf != surfs[menu_info[temp1+string("_surf_over")]])
				{
					l->surf = surfs[menu_info[temp1+string("_surf_over")]];
					l->changed = true;
					result = true;
				}
			} else {
				if (l->surf != surfs[menu_info[temp1+string("_surf")]])
				{
					l->surf = surfs[menu_info[temp1+string("_surf")]];
					l->changed = true;
					result = true;
				}
			}
		}
	}
	return(result);
}

bool Menu::hit_test (int x, int y, SDL_Rect *rect)
{
	if (x >= rect->x && x <= rect->x+rect->w && y >= rect->y && y <= rect->y+rect->h)
	{
		return true;
	} else {
		return false;
	}
}

Menu::Menu (const char *filename, Namedv *data)
{
	if (prefs_file::read(filename, &menu_info) != 0)
	{
		printf("couldn't load menu file: %s\n", filename);
		exit(1);
	}
	background = IMG_Load(menu_info["background"].c_str());
	if (background == NULL)
	{
		printf("couldn't load menu background: %s\n", menu_info["background"].c_str());
		exit(1);
	}
	background = SDL_DisplayFormat(background);
	if (background == NULL)
	{
		printf("couldn't convert menu background to display format\n");
		exit(1);
	}
	ls.add(layer("background", background, 0, 0));
	int num_surfs = menu_info.geti("num_surfs");
	for (int i = 1; i <= num_surfs; i++)
	{
		string key = menu_info[string("surf")+funcs::str_int(i)+string("_name")];
		string path = menu_info[string("surf")+funcs::str_int(i)+string("_path")];
		string alpha = menu_info[string("surf")+funcs::str_int(i)+string("_alpha")];
		if (surfs[key] == NULL)
		{
			SDL_Surface *temp = IMG_Load(path.c_str());
			if (temp == NULL)
			{
				printf("couldn't load menu image %s: %s\n", key.c_str(), path.c_str());
			} else {
				if (alpha == "true")
				{
					surfs[key] = SDL_DisplayFormatAlpha(temp);
					if (alpha == "true")
					SDL_SetAlpha(surfs[key], SDL_SRCALPHA | SDL_RLEACCEL, 0);
				} else {
					surfs[key] = SDL_DisplayFormat(temp);
				}
				SDL_FreeSurface(temp);
				if (surfs[key] == NULL)
				{
					printf("couln't convert %s to display format\n", key.c_str());
				}
			}
		}
	}
	
	int num_fonts = menu_info.geti("num_fonts");
	if (num_fonts > 0) funcs::init_ttf();
	for (int i = 1; i <= num_fonts; i++)
	{
		string key = menu_info[string("font")+funcs::str_int(i)+string("_name")];
		string path = menu_info[string("font")+funcs::str_int(i)+string("_path")];
		int size = menu_info.geti(string("font")+funcs::str_int(i)+string("_size"));
		if (ifstream(path.c_str()).rdstate() == 0 && fonts[key] == 0)
		{
			fonts[key] = TTF_OpenFont(path.c_str(), size);
			if (fonts[key] == NULL)
			{
				printf("couldn't load font \"%s\" from file \"%s\"\n", key.c_str(), path.c_str());
			}
		}
	}
	
	int num_tsurfs = menu_info.geti("num_tsurfs");
	for (int i = 1; i <= num_tsurfs; i++)
	{
		string key = menu_info[string("tsurf")+funcs::str_int(i)+string("_name")];
		string text = add_vars(menu_info[string("tsurf")+funcs::str_int(i)+string("_text")], data);
		string font = menu_info[string("tsurf")+funcs::str_int(i)+string("_font")];
		string color = menu_info[string("tsurf")+funcs::str_int(i)+string("_color")];
		if (surfs[key] == NULL && fonts[font] != NULL)
		{
			surfs[key] = TTF_RenderText_Blended(fonts[font],  text.c_str(), funcs::make_color(color));
		}
	}
	
	int num_buttons = menu_info.geti("num_buttons");
	for (int i = 1; i <= num_buttons; i++)
	{
		string rectStr = menu_info[string("button")+funcs::str_int(i)+string("_rect")];
		SDL_Rect rect;
		if (rectStr != "")
		{
			int start;
			rect.x = atoi(prefs_file::readUntil(rectStr, &start, ',').c_str());
			rectStr = rectStr.substr(start, rectStr.size());
			rect.y = atoi(prefs_file::readUntil(rectStr, &start, ',').c_str());
			rectStr = rectStr.substr(start, rectStr.size());
			rect.w = atoi(prefs_file::readUntil(rectStr, &start, ',').c_str());
			rectStr = rectStr.substr(start, rectStr.size());
			rect.h = atoi(rectStr.c_str());
		} else {
			if (surfs[menu_info[string("button")+funcs::str_int(i)+string("_surf")]] != NULL)
			{
				SDL_GetClipRect(surfs[menu_info[string("button")+funcs::str_int(i)+string("_surf")]], &rect);
			}
		}
		string temp = string("button")+funcs::str_int(i);
		rect.x += menu_info.geti(temp+string("_x"));
		rect.y += menu_info.geti(temp+string("_y"));
		button_rects.push_back(rect);
		ls.add(layer(temp, surfs[menu_info[temp+string("_surf")]], menu_info.geti(temp+string("_x")), menu_info.geti(temp+string("_y"))));
	}
}

string Menu::add_vars (string str, Namedv *data)
{
	if (data != NULL)
	{
		string newstr;
		unsigned int index = 0;
		unsigned int oldindex = 0;
		bool type = 0;
		while ( (index = str.find('%', oldindex)) != string::npos)
		{
			index++;
			if (type == 0)
			{
				newstr += str.substr(oldindex, index-oldindex-1);
			} else {
				newstr += (*data)[str.substr(oldindex, index-oldindex-1)];
			}
			oldindex = index;
			type ? type = 0 : type = 1;
		}
		newstr += str.substr(oldindex);
		return(newstr);
	} else {
		return(str);
	}
}

Menu::~Menu (void)
{
	for (map<string, SDL_Surface*>::iterator itor = surfs.begin(); itor != surfs.end(); ++itor)
	{
		if (itor->second != NULL) SDL_FreeSurface(itor->second);
	}
	SDL_FreeSurface(background);
	
	for (map<string, TTF_Font*>::iterator itor = fonts.begin(); itor != fonts.end(); ++itor)
	{
		if (itor->second != NULL) TTF_CloseFont(itor->second);
	}
}
